%{ open Ast %}

%token PLUS MINUS TIMES DIVIDE SPLIT BY ASSIGN LT LEQ GT GEQ EQUAL NEQ AND OR
%token EOF POUND LPAREN RPAREN LBRACE RBRACE COMMA COLON SEMI
%token INT TEXT DOUBLE BOOL LIST TRUE FALSE
%token MAPPER REDUCER DEF
%token EMIT IF ELSE FOREACH IN
%token JOBNAME

%token <string>ID 
%token <float>DOUBLE_CONST 
%token <int>INTEGER_CONST 
%token <string>TEXT_CONST

%nonassoc NOELSE
%nonassoc IF ELSE FOREACH 
%right ASSIGN
%left AND OR
%left LT LEQ GT GEQ EQUAL NEQ
%left PLUS MINUS
%left TIMES DIVIDE
%nonassoc UMINUS

%start program
%type <Ast.program> program

%%

program: 
	config_dec mapper_def reducer_def	{ Program($1, $2, $3) }
	 
config_dec:
	POUND JOBNAME ASSIGN TEXT_CONST	{ Config_dec("JobName", $4) }

mapper_def:
	DEF ID parameters mapping_relation COLON MAPPER LBRACE stmts RBRACE
		{ {	map_name = $2;
			map_paras = $3;
			map_rel = $4;
			map_body = List.rev $8 } }

reducer_def:
	DEF ID parameters mapping_relation COLON REDUCER LBRACE stmts RBRACE
		{ {	reduce_name = $2;
			red_paras = $3;
			red_rel = $4;
			red_body = List.rev $8 } }

mapping_relation:
	LT LPAREN type_spc COMMA type_spc RPAREN MINUS GT LPAREN type_spc COMMA type_spc RPAREN GT	{ Map_rel($3, $5, $10, $12) }

parameters:
	  LPAREN ID COMMA ID RPAREN	{ $2::$4::[] }

type_spc:
	  INT			{ Atom_type_spc(Int) }
	| DOUBLE		{ Atom_type_spc(Double) }
	| BOOL			{ Atom_type_spc(Bool) }
	| TEXT			{ Atom_type_spc(Text) }
	| LIST LT INT GT	{ List_type_spc(Int) }
	| LIST LT DOUBLE GT	{ List_type_spc(Double) }
	| LIST LT BOOL GT	{ List_type_spc(Bool) }
	| LIST LT TEXT GT	{ List_type_spc(Text) }

stmts:
	/* nothing */	{ [] }
	| stmts stmt { $2 :: $1 }

stmt:	
	  expr SEMI				 { Expr($1) }
	| LBRACE stmts RBRACE			 { Block(List.rev $2) }
	| EMIT LPAREN ID COMMA ID RPAREN SEMI	 { Emit($3, $5) }
	| IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
	| IF LPAREN expr RPAREN stmt ELSE stmt   { If($3, $5, $7) }
	| FOREACH ID IN ID stmt	 { Foreach($2, $4, $5) }

expr:
	  ID		     	  { Id($1) }
	| INTEGER_CONST    	  { Integer_Const($1) }
	| DOUBLE_CONST 	  	  { Double_Const($1) }
	| TEXT_CONST		  { Text_Const($1) }
	| TRUE			  { Bool_Const(true) }
	| FALSE			  { Bool_Const(false) }
	| LPAREN expr RPAREN	  { $2 }	
	| SPLIT ID BY TEXT_CONST  { Split($2, $4) }
	| expr PLUS expr	  { Binop($1, Add, $3) }
	| expr MINUS expr	  { Binop($1, Sub, $3) }
	| expr TIMES expr	  { Binop($1, Mul, $3) }
	| expr DIVIDE expr	  { Binop($1, Div, $3) }
	| MINUS expr %prec UMINUS { Uminus($2) }
	| expr LT expr		  { Binop($1, Lt, $3) }
	| expr LEQ expr		  { Binop($1, Leq, $3) }
	| expr GT expr		  { Binop($1, Gt, $3) }
	| expr GEQ expr		  { Binop($1, Geq, $3) }
	| expr EQUAL expr	  { Binop($1, Equal, $3) }
	| expr NEQ expr		  { Binop($1, Neq, $3) }
	| expr AND expr           { Binop($1, And, $3) }
	| expr OR expr		  { Binop($1, Or, $3) }
	| ID ASSIGN expr	  { Assign($1, $3) }
	| type_spc ID		  { Vdcl($1, $2) }
	| type_spc ID ASSIGN expr { Vdfn($1, $2, $4) }	
