{ open Parser }

let letter = ['a'-'z''A'-'Z']
let digit = ['0'-'9']
let identifier = letter(letter|digit|'_')*

let integer = digit+

let exponent = ['e''E']['+''-']?digit+

let double =
			digit+'.'digit*exponent?
		|	digit*'.'digit+exponent?
		|	digit+'.'digit*exponent
		|	digit*'.'digit+exponent

let text = '"'[^'"''\r''\n']*'"'
(*       '"'['#'-'~'' ''!''\n''\t''\'''\\']*'"' *)

rule token = parse
       [' ' '\t' '\r' '\n'] { token lexbuf }
     | "//" { comment lexbuf }
	 | "JobName" { JOBNAME }
	 | "Text" { TEXT }
     | "Int" { INT }
     | "Double" { DOUBLE }
     | "Boolean" { BOOL }
     | "True" { TRUE }
     | "False" { FALSE }
     | "List" { LIST }
     | "def" { DEF }
     | "Mapper" { MAPPER }
     | "Reducer" { REDUCER }
     | "split" { SPLIT }
     | "by" { BY }
     | "emit" { EMIT }
     | "if" { IF }
     | "else" { ELSE }
     | "foreach" { FOREACH }
     | "in" { IN }
     | '#' { POUND }
     | '(' { LPAREN }
     | ')' { RPAREN }
     | '{' { LBRACE }
     | '}' { RBRACE }
     | ',' { COMMA }
     | ':' { COLON }
     | ';' { SEMI }
     | '+' { PLUS }
     | '-' { MINUS }
     | '*' { TIMES }
     | '/' { DIVIDE }
     | '=' { ASSIGN }
     | '<' { LT }
     | "<=" { LEQ }
     | '>' { GT }
     | ">=" { GEQ }
     | "==" { EQUAL }
     | "!=" { NEQ }
     | "and" { AND }
     | "or" { OR }
     | identifier as var { ID(var) }
     | double as lit { DOUBLE_CONST(float_of_string lit) }
     | integer as lit { INTEGER_CONST(int_of_float(float_of_string lit)) }
     | text as lit { TEXT_CONST(String.sub lit 1 ((String.length lit) - 2 )) }
     | eof { EOF }
     | _ as char { raise (Failure("Lexical Analysis: Illegal Character " ^
       Char.escaped char)) }

and comment = parse
       "\n" { token lexbuf }
     | _    { comment lexbuf}
