open Ast
open Printf

module StringMap = Map.Make(String)
(* get input output *)
let get_input_key = function
Map_rel(input_key, input_value, output_key, output_value) -> input_key

let get_input_value = function
Map_rel(input_key, input_value, output_key, output_value) -> input_value

let get_output_key = function
Map_rel(input_key, input_value, output_key, output_value) -> output_key

let get_output_value = function
Map_rel(input_key, input_value, output_key, output_value) -> output_value

(********************************************)
let add_symbol mr_name mr_type symbol_table =
symbol_table := StringMap.add mr_name mr_type !symbol_table; !symbol_table

let remove_symbol mr_name symbol_table =
symbol_table := StringMap.remove mr_name !symbol_table; !symbol_table

let get_symbol mr_name symbol_table =
	try
		StringMap.find mr_name !symbol_table
	with Not_found -> raise (Failure ("undeclared variable " ^ mr_name))

let get_element_type id symbol_table =
    let mr_type = get_symbol id symbol_table in
match mr_type with
        List_type_spc(Int) -> Atom_type_spc(Int)
    | List_type_spc(Double) -> Atom_type_spc(Double)
    | List_type_spc(Bool) -> Atom_type_spc(Bool)
    | List_type_spc(Text) -> Atom_type_spc(Text)
    | _ -> raise (Failure ( id ^ " is not a List"))


let get_list_type = function
Atom_type_spc(Int) -> List_type_spc(Int)
| Atom_type_spc(Double) -> List_type_spc(Double)
| Atom_type_spc(Bool) -> List_type_spc(Bool)
| Atom_type_spc(Text) -> List_type_spc(Text)
| _ -> raise (Failure ("Unexpected List Type"))

let split_type_checker t1 = match t1 with
	Atom_type_spc(Text) -> ignore(true)
	|_ -> raise (Failure ("expected a Text type"))


let type_of_op op = match op with
	Add -> "Arithmetic"
	|Sub -> "Arithmetic"
	|Div -> "Arithmetic"
	|Mul -> "Artihmetic"
	|Lt -> "Relational"
	|Leq -> "Relational"
	|Gt -> "Relational"
	|Geq -> "Relational"
	|Equal -> "Relational"
	|Neq -> "Relational"
	|And -> "Logical"
	|Or -> "Logical"

let check_op_type t1 op t2 = match (t1, op, t2) with
	(Atom_type_spc(Int), "Arithmetic",  Atom_type_spc(Int)) -> Atom_type_spc(Int)
	|(Atom_type_spc(Double), "Arithmetic", Atom_type_spc(Int)) -> Atom_type_spc(Double)
	|(Atom_type_spc(Int), "Arithmetic", Atom_type_spc(Double)) -> Atom_type_spc(Double)
	|(Atom_type_spc(Double), "Arithmetic", Atom_type_spc(Double)) -> Atom_type_spc(Double)
	|(Atom_type_spc(Int), "Relational", Atom_type_spc(Int)) -> Atom_type_spc(Bool)
	|(Atom_type_spc(Double), "Relational", Atom_type_spc(Int)) -> Atom_type_spc(Bool)
	|(Atom_type_spc(Int), "Relational", Atom_type_spc(Double)) -> Atom_type_spc(Bool)
	|(Atom_type_spc(Double), "Relational", Atom_type_spc(Double)) -> Atom_type_spc(Bool)
	|(Atom_type_spc(Bool), "Logical", Atom_type_spc(Bool)) -> Atom_type_spc(Bool)
	|_ -> raise (Failure ("illegal type of opprand"))


let rec type_of_expr symbol_table = function
	Id(id) -> get_symbol id symbol_table
	|Integer_Const(v) -> Atom_type_spc(Int)
	|Double_Const(v) -> Atom_type_spc(Double)
	|Text_Const(v) -> Atom_type_spc(Text)
	|Bool_Const(v) -> Atom_type_spc(Bool)
	|Split(id, delimitter) -> ignore(let mr_type = get_symbol id symbol_table in split_type_checker mr_type); List_type_spc(Text)
	|Binop(expr1, op, expr2) -> let type1 = type_of_expr symbol_table expr1 	and type2 = type_of_expr symbol_table expr2 
	and op_type = type_of_op op in 
		check_op_type type1 op_type type2 
	|_-> raise (Failure("Not supported"))

 let assign_checker id expr symbol_table =
    	let id_type = get_symbol id symbol_table in
        let expr_type = type_of_expr symbol_table expr in
        if (id_type = expr_type) then ignore(true)
        else raise (Failure ("illegal assign type"))


	
let emit_checker t1 t2 symbol_table = 
	let out_key = get_symbol "out_key_type" symbol_table in
	let out_value = get_symbol "out_value_type" symbol_table in
	if(t1 = out_key && t2 = out_value) then ignore(true)
	else raise (Failure ("illegal emit type"))
		

let check_if expr symbol_table =
	let if_type = type_of_expr symbol_table expr in
	match if_type with
	Atom_type_spc(Bool) -> ignore(true)
	|_ -> raise (Failure ("illegal if type"))

let check_vdfn id expr symbol_table =
	let expr_type = type_of_expr symbol_table expr in
	let id_type = get_symbol id symbol_table in
	if (expr_type = id_type) then ignore(true) 
	else raise (Failure ("illegal vdfn type"))
(* **************************)
let rec typechecking_expr symbol_table = function
        Vdfn(type_spc, id, expr) -> ignore(add_symbol id type_spc symbol_table); check_vdfn id expr symbol_table
	|Vdcl(type_spc, id) -> ignore(add_symbol id type_spc symbol_table) 
 	|Split(id, delimitter) -> let mr_type = get_symbol id symbol_table in split_type_checker mr_type  
	|Assign(id, expr) -> assign_checker id expr symbol_table
	|_-> raise (Failure ("Not supported"))
let rec typechecking_stmt symbol_table = function
	Block(statements) -> ignore(typechecking_stmts symbol_table statements); symbol_table
	|Expr(expr) -> (typechecking_expr symbol_table expr); symbol_table 
	|Foreach(id1, id2, stmt) -> let element_type = get_element_type id2 symbol_table in ignore(add_symbol id1 element_type symbol_table); ignore(typechecking_stmt symbol_table stmt); symbol_table
	|Emit(id1, id2) -> let key_type = get_symbol id1 symbol_table in
	let value_type = get_symbol id2 symbol_table in emit_checker key_type value_type symbol_table; symbol_table
	|If(expr, stmt1, stmt2) -> check_if expr symbol_table; ignore(typechecking_stmt symbol_table stmt1); ignore(typechecking_stmt symbol_table stmt2); symbol_table
	
and typechecking_stmts symbol_table statements = 
List.fold_left typechecking_stmt symbol_table statements

let check_reducer (reducer:Ast.reducer_def) = 
	let symbol_table = ref StringMap.empty in
        let input_key_type = get_input_key reducer.red_rel in
        let input_value_type = get_input_value reducer.red_rel in
        let output_key_type = get_output_key reducer.red_rel in
        let output_value_type = get_output_value reducer.red_rel in
        let para1 = List.nth reducer.red_paras 0 in
        let para2 = List.nth reducer.red_paras 1 in
        let _ = add_symbol para1 input_key_type symbol_table in
        let _ = add_symbol para2 (get_list_type input_value_type) symbol_table in
	let _ = add_symbol "out_key_type" output_key_type symbol_table in
	let _ = add_symbol "out_value_type" output_value_type symbol_table in
        (typechecking_stmts symbol_table reducer.red_body)


let check_mapper (mapper:Ast.mapper_def) = 
	let symbol_table = ref StringMap.empty in
	let input_key_type = get_input_key mapper.map_rel in
	let input_value_type = get_input_value mapper.map_rel in
	let output_key_type = get_output_key mapper.map_rel in
	let output_value_type = get_output_value mapper.map_rel in
	let para1 = List.nth mapper.map_paras 0 in
	let para2 = List.nth mapper.map_paras 1 in
	let _ = add_symbol para1 input_key_type symbol_table in
	let _ = add_symbol para2 input_value_type symbol_table in
	let _ = add_symbol "out_key_type" output_key_type symbol_table in
	let _ = add_symbol "out_value_type" output_value_type symbol_table in
	(typechecking_stmts symbol_table mapper.map_body)
 
let rec typechecker = function
	Program(configuration, mapper, reducer) ->
	ignore(check_mapper mapper);
	print_string "Mapper checked\n";
	ignore(check_reducer reducer);
	print_string "Reducer checked\n"

(* main *)
